@@ -4,6 +4,7 @@ from _testcapi import traceback_print
 from StringIO import StringIO
 import sys
 import unittest
+from imp import reload
 from test.test_support import run_unittest, is_jython, Error
 
 import traceback
@@ -34,6 +35,9 @@ class TracebackCases(unittest.TestCase):
     def syntax_error_with_caret(self):
         compile("def fact(x):\n\treturn x!\n", "?", "exec")
 
+    def syntax_error_with_caret_2(self):
+        compile("1 +\n", "?", "exec")
+
     def syntax_error_without_caret(self):
         # XXX why doesn't compile raise the same traceback?
         import test.badsyntax_nocaret
@@ -49,6 +53,12 @@ class TracebackCases(unittest.TestCase):
         self.assert_("^" in err[2]) # third line has caret
         self.assert_(err[1].find("!") == err[2].find("^")) # in the right place
 
+        err = self.get_exception_format(self.syntax_error_with_caret_2,
+                                        SyntaxError)
+        self.assert_("^" in err[2]) # third line has caret
+        self.assert_(err[2].count('\n') == 1) # and no additional newline
+        self.assert_(err[1].find("+") == err[2].find("^")) # in the right place
+
     def test_nocaret(self):
         if is_jython:
             # jython adds a caret in this case (why shouldn't it?)
@@ -149,7 +159,7 @@ def test():
     def test_format_exception_only_bad__str__(self):
         class X(Exception):
             def __str__(self):
-                1/0
+                1 // 0
         err = traceback.format_exception_only(X, X())
         self.assertEqual(len(err), 1)
         str_value = '<unprintable %s object>' % X.__name__
@@ -159,6 +169,15 @@ def test():
         err = traceback.format_exception_only(None, None)
         self.assertEqual(err, ['None\n'])
 
+    def test_unicode(self):
+        err = AssertionError('\xff')
+        lines = traceback.format_exception_only(type(err), err)
+        self.assertEqual(lines, ['AssertionError: \xff\n'])
+
+        err = AssertionError(u'\xe9')
+        lines = traceback.format_exception_only(type(err), err)
+        self.assertEqual(lines, ['AssertionError: \\xe9\n'])
+
 
 class TracebackFormatTests(unittest.TestCase):
 
